==> hourglass-1.0.0.tar.gz: hourglass-1.0.0/src/print.c <==
/*
 *
 * Copyright (c) 2001 University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Hourglass.
 * 
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation is hereby granted without fee, provided that the
 * above copyright notice and this permission/disclaimer notice is
 * retained in all copies or modified versions, and that both notices
 * appear in supporting documentation.  THE COPYRIGHT HOLDERS PROVIDE
 * THIS SOFTWARE "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE COPYRIGHT
 * HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR ANY DAMAGES WHATSOEVER
 * RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu
 * any improvements that they make and grant redistribution rights to the
 * University of Utah.
 * 
 * Author: John Regehr (regehr@cs.utah.edu)
 */

#include "hourglass.h"

int suppress_trace = 0;

static void print_results_lat (int id) 
{
  int j;
  sec_time dur;

  fprintf (stdout, "microseconds late\n");
  for (j=1; j<thrd_data[id].lat_slot; j++) {
    fprintf (stdout, "latlate: %f\n", 
	     cycle_to_us(thrd_data[id].lat_slots[j] -
			 thrd_data[id].lat_slots[j-1]) -
	     thrd_data[id].app_period);
  }
  dur = cycle_to_sec (end_time - start_time);
  fprintf (stderr, "%d slots filled; %f per second\n", 
	   thrd_data[id].lat_slot, 
	   thrd_data[id].lat_slot / dur);
}

static void print_deadlines (int id) 
{
  printf ("thread %d: missed %d deadlines, hit %d\n", 
	  id, thrd_data[id].misses, thrd_data[id].hits);
}

cycle_time trace_start;
cycle_time trace_end;

void print_gap_hist (void)
{
  int i, total, cum;
  double pcts[] = { 0.5, 0.9, 0.99, 0.999, 0.9999, 0.99999, 0.999999, 0.9999999 };
  int pcts_printed[] = { 0, 0, 0, 0, 0, 0, 0, 0 };
  int median = 0;

  total = 0;
  for (i=0; i<NUM_GAP_HIST_BUCKETS; i++) {
    total += gap_hist[i];
  }
  total += gap_hist_toobig;

  printf ("\ngap histogram in timestamp counter ticks:\n");
  cum = 0;
  for (i=0; i<NUM_GAP_HIST_BUCKETS; i++) {
    if (gap_hist[i]) {
      int j;

      printf ("  %d - %d : %d\n",
	      i*GAP_HIST_DIVISOR,
	      ((i+1)*GAP_HIST_DIVISOR)-1,
	      gap_hist[i]);

      cum += gap_hist[i];

      for (j=0; j<8; j++) {
	if (!pcts_printed[j] &&
	    (double)cum >= (pcts[j]*total)) {
	  pcts_printed[j] = 1;
	  printf ("%f%% of samples were smaller than this; ",
		  pcts[j]*100);
	  printf ("to cutoff here, call hourglass with -g %fus\n",
		  cycle_to_us(((cycle_time)i+1)*GAP_HIST_DIVISOR));
	  if (pcts[j] == 0.5) {
	    median = (i+1)*GAP_HIST_DIVISOR;
	  }
	}
      }
    }
  }
  printf ("  %d gaps were too big to record in the histogram\n", gap_hist_toobig);
  printf ("\n");
  printf ("  median cost of running the polling loop was approx %fus\n",
	  cycle_to_us ((cycle_time)median));
  printf ("  this would probably be a good value for the polling overhead\n");
}  

static void print_trace_recs (void)
{
  int i, j;
  cycle_time dur;
  cycle_time total_thr_time = 0;

  printf ("there were %ld out of %d trace records; %3f %% used.\n",
	  current_record, NUM_TRACE_RECORDS,
	  100 * (1.0 * current_record / NUM_TRACE_RECORDS));
 
  for (i=1; i<current_record; i++) {
    if (trace_start > trace_recs[i].start &&
	trace_recs[i].start != 0) {
      trace_start = trace_recs[i].start;
    }
    if (trace_end < trace_recs[i].end &&
	trace_recs[i].end != 0) {
      trace_end = trace_recs[i].end;
    }
  }

  printf ("in cycles, trace start %lld, end %lld, duration %lld\n", 
	  trace_start, trace_end, trace_end - trace_start);

  dur = trace_end - trace_start;
  printf ("trace duration %f seconds\n", cycle_to_sec (dur));

  for (j=0; j<num_threads; j++) {
    thrd_data[j].thread_time = 0;
    for (i=1; i<current_record; i++) {
      if (trace_recs[i].id == j && trace_recs[i].end != 0) {
	thrd_data[j].thread_time += trace_recs[i].end - trace_recs[i].start;
      }
    }
    printf ("thread %d recorded %f seconds (%f %%)\n", 
	    j, 
	    cycle_to_sec (thrd_data[j].thread_time),
	    cycle_to_sec (thrd_data[j].thread_time) / cycle_to_sec (dur) * 100);
    total_thr_time += thrd_data[j].thread_time;
  }
  printf ("total thread time %f seconds\n", cycle_to_sec (total_thr_time));

  if (!suppress_trace) {
    printf ("\ntime slots in ms: thread start end duration gap:\n");
    for (i=2; i<current_record; i++) {
      if (trace_recs[i].end != 0) {
	// FOO
	printf ("tracerec: %d %f %f %f %f\n",
		trace_recs[i].id,
		1000 * cycle_to_sec (trace_recs[i].start - trace_start),
		1000 * cycle_to_sec (trace_recs[i].end - trace_start),
		1000 * cycle_to_sec (trace_recs[i].end - trace_recs[i].start),
		1000 * cycle_to_sec (trace_recs[i].start - trace_recs[i-1].end));
      }
    }
  }

  if (generate_gap_hist) {
    print_gap_hist ();
  }
}
	 
static void print_trace_recs_cycles (void)
{
  int i;

  printf ("\ntime slots in cycles: thread start end:\n");
  for (i=1; i<current_record; i++) {
    if (trace_recs[i].end != 0) {
      printf ("tracereccyc: %d %lld %lld\n",
	      trace_recs[i].id,
	      trace_recs[i].start,
	      trace_recs[i].end);
    }
  }
}
	 
void print_all_results (void)
{
  int i;

  printf ("numthreads: %d\n", num_threads);

  for (i=0; i<num_threads; i++) {
    
    printf ("work done by thrd %d : %lld\n", 
	    i, thrd_data[i].workdone);

    switch (thrd_data[i].work) {

    case LAT:
      print_results_lat (i);
      break;

    case PERIODIC:
    case CPU_PERIODIC:
      print_deadlines (i);
      break;

    case CPU:
    case CPU_SCAN:
    case CPU_SCAN_YIELD:
    case CPU_YIELD:
      break;

    default:
      printf ("Hrm: unexpected thread workload -- aborting\n");
      exit (-1);
    }
  }

  if (tracing) {
    print_trace_recs ();
    if (print_cycles) {
      print_trace_recs_cycles ();
    }
  }
}

==> hourglass-1.0.1b.tar.gz: hourglass-1.0.1b/src/print.c <==
/*
 *
 * Copyright (c) 2001 University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Hourglass.
 * 
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation is hereby granted without fee, provided that the
 * above copyright notice and this permission/disclaimer notice is
 * retained in all copies or modified versions, and that both notices
 * appear in supporting documentation.  THE COPYRIGHT HOLDERS PROVIDE
 * THIS SOFTWARE "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE COPYRIGHT
 * HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR ANY DAMAGES WHATSOEVER
 * RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu
 * any improvements that they make and grant redistribution rights to the
 * University of Utah.
 * 
 * Author: John Regehr (regehr@cs.utah.edu)
 */

#include "hourglass.h"

int suppress_trace = 0;

static void print_results_lat (int id) 
{
  int j;
  sec_time dur;

  fprintf (stdout, "microseconds late\n");
  for (j=1; j<thrd_data[id].lat_slot; j++) {
    fprintf (stdout, "latlate: %f\n", 
	     cycle_to_us(thrd_data[id].lat_slots[j] -
			 thrd_data[id].lat_slots[j-1]) -
	     thrd_data[id].app_period);
  }
  dur = cycle_to_sec (end_time - start_time);
  fprintf (stderr, "%d slots filled; %f per second\n", 
	   thrd_data[id].lat_slot, 
	   thrd_data[id].lat_slot / dur);
}

static void print_deadlines (int id) 
{
  printf ("thread %d: missed %d deadlines, hit %d\n", 
	  id, thrd_data[id].misses, thrd_data[id].hits);
}

cycle_time trace_start;
cycle_time trace_end;

void print_gap_hist (void)
{
  int i, total, cum;
  double pcts[] = { 0.5, 0.9, 0.99, 0.999, 0.9999, 0.99999, 0.999999, 0.9999999 };
  int pcts_printed[] = { 0, 0, 0, 0, 0, 0, 0, 0 };
  int median = 0;

  total = 0;
  for (i=0; i<NUM_GAP_HIST_BUCKETS; i++) {
    total += gap_hist[i];
  }
  total += gap_hist_toobig;

  printf ("\ngap histogram in timestamp counter ticks:\n");
  cum = 0;
  for (i=0; i<NUM_GAP_HIST_BUCKETS; i++) {
    if (gap_hist[i]) {
      int j;

      printf ("  %d - %d : %d\n",
	      i*GAP_HIST_DIVISOR,
	      ((i+1)*GAP_HIST_DIVISOR)-1,
	      gap_hist[i]);

      cum += gap_hist[i];

      for (j=0; j<8; j++) {
	if (!pcts_printed[j] &&
	    (double)cum >= (pcts[j]*total)) {
	  pcts_printed[j] = 1;
	  printf ("%f%% of samples were smaller than this; ",
		  pcts[j]*100);
	  printf ("to cutoff here, call hourglass with -g %fus\n",
		  cycle_to_us(((cycle_time)i+1)*GAP_HIST_DIVISOR));
	  if (pcts[j] == 0.5) {
	    median = (i+1)*GAP_HIST_DIVISOR;
	  }
	}
      }
    }
  }
  printf ("  %d gaps were too big to record in the histogram\n", gap_hist_toobig);
  printf ("\n");
  printf ("  median cost of running the polling loop was approx %fus\n",
	  cycle_to_us ((cycle_time)median));
  printf ("  this would probably be a good value for the polling overhead\n");
}  

static void print_trace_recs (void)
{
  int i, j;
  cycle_time dur;
  cycle_time total_thr_time = 0;

  printf ("there were %ld out of %d trace records; %3f %% used.\n",
	  current_record, NUM_TRACE_RECORDS,
	  100 * (1.0 * current_record / NUM_TRACE_RECORDS));
 
  for (i=1; i<current_record; i++) {
    if (trace_start > trace_recs[i].start &&
	trace_recs[i].start != 0) {
      trace_start = trace_recs[i].start;
    }
    if (trace_end < trace_recs[i].end &&
	trace_recs[i].end != 0) {
      trace_end = trace_recs[i].end;
    }
  }

  printf ("in cycles, trace start %lld, end %lld, duration %lld\n", 
	  trace_start, trace_end, trace_end - trace_start);

  dur = trace_end - trace_start;
  printf ("trace duration %f seconds\n", cycle_to_sec (dur));

  for (j=0; j<num_threads; j++) {
    thrd_data[j].thread_time = 0;
    for (i=1; i<current_record; i++) {
      if (trace_recs[i].id == j && trace_recs[i].end != 0) {
	thrd_data[j].thread_time += trace_recs[i].end - trace_recs[i].start;
      }
    }
    printf ("thread %d recorded %f seconds (%f %%)\n", 
	    j, 
	    cycle_to_sec (thrd_data[j].thread_time),
	    cycle_to_sec (thrd_data[j].thread_time) / cycle_to_sec (dur) * 100);
    total_thr_time += thrd_data[j].thread_time;
  }
  printf ("total thread time %f seconds\n", cycle_to_sec (total_thr_time));

  if (!suppress_trace) {
    printf ("\ntime slots in ms: thread start end duration gap:\n");
    for (i=2; i<current_record; i++) {
      if (trace_recs[i].end != 0) {
	// FOO
	printf ("tracerec: %d %f %f %f %f\n",
		trace_recs[i].id,
		1000 * cycle_to_sec (trace_recs[i].start - trace_start),
		1000 * cycle_to_sec (trace_recs[i].end - trace_start),
		1000 * cycle_to_sec (trace_recs[i].end - trace_recs[i].start),
		1000 * cycle_to_sec (trace_recs[i].gap)); //rjd -- changed to SMP aware code (trace_recs[i].start - trace_recs[i-1].end));
      }
    }
  }

  if (generate_gap_hist) {
    print_gap_hist ();
  }
}
	 
static void print_trace_recs_cycles (void)
{
  int i;

  printf ("\ntime slots in cycles: thread start end:\n");
  for (i=1; i<current_record; i++) {
    if (trace_recs[i].end != 0) {
      printf ("tracereccyc: %d %lld %lld\n",
	      trace_recs[i].id,
	      trace_recs[i].start,
	      trace_recs[i].end);
    }
  }
}
	 
void print_all_results (void)
{
  int i;

  printf ("numthreads: %d\n", num_threads);

  for (i=0; i<num_threads; i++) {
    
    printf ("work done by thrd %d : %lld\n", 
	    i, thrd_data[i].workdone);

    switch (thrd_data[i].work) {

    case LAT:
      print_results_lat (i);
      break;

    case PERIODIC:
    case CPU_PERIODIC:
      print_deadlines (i);
      break;

    case CPU:
    case CPU_SCAN:
    case CPU_SCAN_YIELD:
    case CPU_YIELD:
    case IO_DUMB:
      break;

    default:
      printf ("Hrm: unexpected thread workload -- aborting\n");
      exit (-1);
    }
  }

  if (tracing) {
    print_trace_recs ();
    if (print_cycles) {
      print_trace_recs_cycles ();
    }
  }
}
